package com.icing.dialer

import android.util.Base64
import com.southernstorm.noise.protocol.CipherState
import com.southernstorm.noise.protocol.CipherStatePair
import com.southernstorm.noise.protocol.HandshakeState
import com.southernstorm.noise.protocol.Noise
import javax.crypto.BadPaddingException
import javax.crypto.ShortBufferException
import java.security.NoSuchAlgorithmException
import java.util.Arrays

class NoiseHandler(
    private val localKeyBase64: String, // ED25519 private (initiator) or public (responder) key (Base64-encoded)
    private val remotePublicKeyBase64: String // Remote ED25519 public key (Base64-encoded)
) {
    private var handshakeState: HandshakeState? = null
    private var cipherStatePair: CipherStatePair? = null

    /**
     * Wipes sensitive data by filling the byte array with zeros.
     */
    private fun wipe(data: ByteArray?) {
        data?.let { Arrays.fill(it, 0.toByte()) }
    }

    /**
     * Initializes the Noise handshake.
     * @param isInitiator True if this is the initiator, false if responder.
     * @return The initial handshake message.
     * @throws IllegalArgumentException If keys are invalid.
     * @throws IllegalStateException If handshake fails to start.
     */
    fun initialize(isInitiator: Boolean): ByteArray {
        var localKey: ByteArray? = null
        var remotePublicKey: ByteArray? = null
        try {
            val protocolName = "Noise_IK_25519_AESGCM_SHA256"
            handshakeState = HandshakeState(
                protocolName,
                if (isInitiator) HandshakeState.INITIATOR else HandshakeState.RESPONDER
            )

            // Set local key (private for initiator, public for responder)
            localKey = Base64.decode(localKeyBase64, Base64.DEFAULT)
            if (localKey.size != 32) {
                throw IllegalArgumentException("Invalid local key size: ${localKey.size}")
            }
            if (isInitiator) {
                handshakeState?.localKeyPair?.setPrivateKey(localKey, 0)
                    ?: throw IllegalStateException("Local key pair not initialized")
            } else {
                handshakeState?.localKeyPair?.setPublicKey(localKey, 0)
                    ?: throw IllegalStateException("Local key pair not initialized")
            }

            // Set remote public key
            remotePublicKey = Base64.decode(remotePublicKeyBase64, Base64.DEFAULT)
            if (remotePublicKey.size != 32) {
                throw IllegalArgumentException("Invalid remote public key size: ${remotePublicKey.size}")
            }
            handshakeState?.remotePublicKey?.setPublicKey(remotePublicKey, 0)
                ?: throw IllegalStateException("Remote public key not initialized")

            // Start handshake and write initial message
            handshakeState?.start() ?: throw IllegalStateException("Handshake state not initialized")
            val messageBuffer = ByteArray(256) // Sufficient for IK initial message
            val payload = ByteArray(0) // Empty payload
            val writtenLength: Int = handshakeState?.writeMessage(
                messageBuffer, 0, payload, 0, payload.size
            ) ?: throw IllegalStateException("Failed to write handshake message")
            return messageBuffer.copyOf(writtenLength)
        } catch (e: NoSuchAlgorithmException) {
            throw IllegalStateException("Unsupported algorithm: ${e.message}", e)
        } catch (e: ShortBufferException) {
            throw IllegalStateException("Buffer too small for handshake message", e)
        } finally {
            wipe(localKey)
            wipe(remotePublicKey)
        }
    }

    /**
     * Processes a handshake message and returns the next message or null if complete.
     * @param message The received handshake message.
     * @return The next handshake message or null if handshake is complete.
     * @throws IllegalStateException If handshake state is invalid.
     * @throws BadPaddingException If message decryption fails.
     */
    fun processHandshakeMessage(message: ByteArray): ByteArray? {
        try {
            val handshake = handshakeState ?: throw IllegalStateException("Handshake not initialized")
            val messageBuffer = ByteArray(256) // Sufficient for IK payload + MAC
            val writtenLength: Int = handshake.readMessage(
                message, 0, message.size, messageBuffer, 0
            )

            if (handshake.getAction() == HandshakeState.SPLIT) {
                cipherStatePair = handshake.split()
                return null // Handshake complete
            }

            // Write next message
            val payload = ByteArray(0) // Empty payload
            val nextMessage = ByteArray(256)
            val nextWrittenLength: Int = handshake.writeMessage(
                nextMessage, 0, payload, 0, payload.size
            )
            return nextMessage.copyOf(nextWrittenLength)
        } catch (e: ShortBufferException) {
            throw IllegalStateException("Buffer too small for handshake message", e)
        } catch (e: BadPaddingException) {
            throw IllegalStateException("Invalid handshake message: ${e.message}", e)
        }
    }

    /**
     * Encrypts data using the sender's cipher state.
     * @param data The data to encrypt.
     * @return The encrypted data.
     * @throws IllegalStateException If handshake is not completed.
     */
    fun encryptData(data: ByteArray): ByteArray {
        val cipherState = cipherStatePair?.getSender()
            ?: throw IllegalStateException("Handshake not completed")
        try {
            val outputBuffer = ByteArray(data.size + cipherState.getMACLength()) // Account for AES-GCM MAC
            val length: Int = cipherState.encryptWithAd(null, data, 0, outputBuffer, 0, data.size)
            return outputBuffer.copyOf(length)
        } catch (e: ShortBufferException) {
            throw IllegalStateException("Buffer too small for encryption: ${e.message}", e)
        }
    }

    /**
     * Decrypts data using the receiver's cipher state.
     * @param data The encrypted data.
     * @return The decrypted data.
     * @throws IllegalStateException If handshake is not completed.
     * @throws BadPaddingException If decryption fails.
     */
    fun decryptData(data: ByteArray): ByteArray {
        val cipherState = cipherStatePair?.getReceiver()
            ?: throw IllegalStateException("Handshake not completed")
        try {
            val outputBuffer = ByteArray(data.size)
            val length: Int = cipherState.decryptWithAd(null, data, 0, outputBuffer, 0, data.size)
            return outputBuffer.copyOf(length)
        } catch (e: ShortBufferException) {
            throw IllegalStateException("Buffer too small for decryption: ${e.message}", e)
        } catch (e: BadPaddingException) {
            throw IllegalStateException("Invalid ciphertext: ${e.message}", e)
        }
    }

    /**
     * Cleans up sensitive cryptographic data.
     */
    fun destroy() {
        handshakeState?.destroy()
        cipherStatePair?.destroy()
        handshakeState = null
        cipherStatePair = null
    }
}