package com.icing.dialer.modem

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import androidx.core.app.ActivityCompat
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.collect

class FSKModemExample(private val context: Context) {
    private val modem = FSKModem()
    private val scope = CoroutineScope(Dispatchers.Main + SupervisorJob())
    
    fun checkPermissions(): Boolean {
        return ActivityCompat.checkSelfPermission(
            context,
            Manifest.permission.RECORD_AUDIO
        ) == PackageManager.PERMISSION_GRANTED
    }
    
    fun startModem() {
        if (!checkPermissions()) {
            println("Audio recording permission not granted")
            return
        }
        
        // Initialize modem
        modem.initialize()
        
        // Set up data reception handler
        scope.launch {
            modem.receivedData.collect { data ->
                handleReceivedData(data)
            }
        }
        
        // Monitor modem state
        scope.launch {
            modem.state.collect { state ->
                println("Modem state: $state")
            }
        }
        
        // Start modem
        modem.start()
    }
    
    fun sendTextMessage(message: String) {
        val data = message.toByteArray(Charsets.UTF_8)
        modem.sendData(data)
    }
    
    fun sendBinaryData(data: ByteArray) {
        // Split large data into frames if necessary
        val maxPayloadSize = 200 // Based on Reed-Solomon configuration
        
        for (i in data.indices step maxPayloadSize) {
            val chunk = data.sliceArray(
                i until minOf(i + maxPayloadSize, data.size)
            )
            modem.sendData(chunk)
        }
    }
    
    private fun handleReceivedData(data: ByteArray) {
        // Handle received data
        try {
            val message = String(data, Charsets.UTF_8)
            println("Received message: $message")
        } catch (e: Exception) {
            println("Received binary data: ${data.size} bytes")
        }
    }
    
    fun stopModem() {
        modem.stop()
        modem.release()
        scope.cancel()
    }
    
    // Example: Voice communication
    fun startVoiceTransmission() {
        scope.launch(Dispatchers.IO) {
            // In a real implementation, you would capture audio from microphone
            // and send it through the modem
            while (isActive) {
                // Simulated audio data (replace with actual audio capture)
                val audioFrame = ShortArray(FSKConstants.OPUS_FRAME_SIZE)
                // Fill audioFrame with audio samples...
                
                // Convert to byte array and send
                val audioBytes = audioFrame.toByteArray()
                modem.sendData(audioBytes)
                
                delay(20) // 20ms frames
            }
        }
    }
    
    private fun ShortArray.toByteArray(): ByteArray {
        val bytes = ByteArray(size * 2)
        for (i in indices) {
            bytes[i * 2] = (this[i].toInt() and 0xFF).toByte()
            bytes[i * 2 + 1] = ((this[i].toInt() shr 8) and 0xFF).toByte()
        }
        return bytes
    }
}

// Unit tests
class FSKModemTest {
    fun testModulation() {
        val modulator = FSKModulator()
        
        // Test data
        val testData = byteArrayOf(0x55, 0xAA.toByte(), 0x0F, 0xF0.toByte())
        
        // Convert to symbols
        val symbols = modulator.bytesToSymbols(testData)
        println("Symbols: ${symbols.joinToString()}")
        
        // Generate preamble
        val preamble = modulator.generatePreamble()
        println("Preamble length: ${preamble.size} samples")
        
        // Modulate symbols
        val modulated = modulator.modulateSymbols(symbols)
        println("Modulated signal length: ${modulated.size} samples")
        
        // Apply filter
        val filtered = modulator.applyRaisedCosineFilter(modulated)
        println("Filtered signal length: ${filtered.size} samples")
    }
    
    fun testDemodulation() {
        val modulator = FSKModulator()
        val demodulator = FSKDemodulator()
        
        // Test data
        val testData = "Hello FSK Modem!".toByteArray()
        
        // Modulate
        val symbols = modulator.bytesToSymbols(testData)
        val preamble = modulator.generatePreamble()
        val modulated = modulator.modulateSymbols(symbols)
        
        // Combine preamble and data
        val signal = FloatArray(preamble.size + modulated.size)
        System.arraycopy(preamble, 0, signal, 0, preamble.size)
        System.arraycopy(modulated, 0, signal, preamble.size, modulated.size)
        
        // Find preamble
        val preambleIndex = demodulator.findPreamble(signal)
        println("Preamble found at index: $preambleIndex")
        
        // Demodulate
        val dataStart = preambleIndex + preamble.size
        val dataSignal = signal.sliceArray(dataStart until signal.size)
        val demodSymbols = demodulator.demodulateSamples(dataSignal)
        
        // Convert back to bytes
        val demodData = demodulator.symbolsToBytes(demodSymbols)
        val demodMessage = String(demodData, 0, testData.size)
        println("Demodulated message: $demodMessage")
    }
    
    fun testFrameProcessing() {
        val processor = FrameProcessor()
        
        // Test data
        val testData = ByteArray(200) { it.toByte() }
        
        // Create frame
        val frame = processor.createFrame(testData)
        println("Frame size: ${frame.toByteArray().size} bytes")
        
        // Simulate transmission (no errors)
        val frameBytes = frame.toByteArray()
        
        // Process received frame
        val decoded = processor.processFrame(frameBytes)
        println("Decoded data: ${decoded?.size} bytes")
        
        // Verify data integrity
        if (decoded != null && decoded.contentEquals(testData)) {
            println("Frame processing successful!")
        }
    }
    
    fun testAudioCodec() {
        try {
            val codec = AudioCodec()
            
            // Test audio data (sine wave)
            val testAudio = ShortArray(FSKConstants.OPUS_FRAME_SIZE) { i ->
                (32767 * kotlin.math.sin(2 * kotlin.math.PI * 440 * i / FSKConstants.SAMPLE_RATE)).toInt().toShort()
            }
            
            // Encode
            val encoded = codec.encode(testAudio)
            println("Encoded size: ${encoded.size} bytes (compression ratio: ${testAudio.size * 2.0 / encoded.size})")
            
            // Decode
            val decoded = codec.decode(encoded)
            println("Decoded samples: ${decoded.size}")
            
            // Test packet loss handling
            val lostPacket = codec.decodeLost()
            println("Lost packet recovery: ${lostPacket.size} samples")
            
            codec.release()
        } catch (e: Exception) {
            println("Audio codec test failed: ${e.message}")
        }
    }
}