package com.icing.dialer.modem

import kotlin.math.*

class FSKModulator {
    private var phase = 0.0
    private val symbolBuffer = mutableListOf<Int>()
    
    // Generate preamble for synchronization
    fun generatePreamble(): FloatArray {
        val samples = FloatArray(FSKConstants.PREAMBLE_LENGTH * FSKConstants.SAMPLES_PER_SYMBOL)
        var sampleIndex = 0
        
        // Alternating 01 10 pattern for easy detection
        for (i in 0 until FSKConstants.PREAMBLE_LENGTH) {
            val symbol = if (i % 2 == 0) 1 else 2 // Alternating 01 and 10
            val freq = getFrequencyForSymbol(symbol)
            
            for (j in 0 until FSKConstants.SAMPLES_PER_SYMBOL) {
                samples[sampleIndex++] = generateSample(freq)
            }
        }
        
        return samples
    }
    
    // Convert bytes to 4FSK symbols (2 bits per symbol)
    fun bytesToSymbols(data: ByteArray): IntArray {
        val symbols = IntArray(data.size * 4) // 4 symbols per byte
        var symbolIndex = 0
        
        for (byte in data) {
            val value = byte.toInt() and 0xFF
            // Extract 2-bit symbols from MSB to LSB
            symbols[symbolIndex++] = (value shr 6) and 0x03
            symbols[symbolIndex++] = (value shr 4) and 0x03
            symbols[symbolIndex++] = (value shr 2) and 0x03
            symbols[symbolIndex++] = value and 0x03
        }
        
        return symbols
    }
    
    // Modulate symbols to audio samples with smooth transitions
    fun modulateSymbols(symbols: IntArray): FloatArray {
        val samples = FloatArray(symbols.size * FSKConstants.SAMPLES_PER_SYMBOL)
        var sampleIndex = 0
        
        for (i in symbols.indices) {
            val currentFreq = getFrequencyForSymbol(symbols[i])
            val nextFreq = if (i < symbols.size - 1) {
                getFrequencyForSymbol(symbols[i + 1])
            } else {
                currentFreq
            }
            
            // Generate samples with smooth frequency transition
            for (j in 0 until FSKConstants.SAMPLES_PER_SYMBOL) {
                val progress = j.toFloat() / FSKConstants.SAMPLES_PER_SYMBOL
                val freq = if (j >= FSKConstants.SAMPLES_PER_SYMBOL - 2) {
                    // Smooth transition in last 2 samples
                    currentFreq * (1 - progress) + nextFreq * progress
                } else {
                    currentFreq
                }
                
                samples[sampleIndex++] = generateSample(freq)
            }
        }
        
        return samples
    }
    
    // Generate single sample with continuous phase
    private fun generateSample(frequency: Double): Float {
        val sample = sin(2.0 * PI * phase).toFloat()
        phase += frequency / FSKConstants.SAMPLE_RATE
        
        // Keep phase in [0, 1] range to prevent precision loss
        if (phase >= 1.0) {
            phase -= 1.0
        }
        
        return sample
    }
    
    // Map symbol to frequency
    private fun getFrequencyForSymbol(symbol: Int): Double {
        return when (symbol) {
            0 -> FSKConstants.FREQ_00
            1 -> FSKConstants.FREQ_01
            2 -> FSKConstants.FREQ_10
            3 -> FSKConstants.FREQ_11
            else -> throw IllegalArgumentException("Invalid symbol: $symbol")
        }
    }
    
    // Apply raised cosine filter for spectral shaping
    fun applyRaisedCosineFilter(samples: FloatArray): FloatArray {
        val alpha = 0.35 // Roll-off factor
        val filteredSamples = FloatArray(samples.size)
        val filterLength = 65 // Filter taps
        val halfLength = filterLength / 2
        
        for (i in samples.indices) {
            var sum = 0.0f
            
            for (j in -halfLength..halfLength) {
                val sampleIndex = i + j
                if (sampleIndex in samples.indices) {
                    val t = j.toFloat() / FSKConstants.SAMPLES_PER_SYMBOL
                    val h = if (abs(t) < 1e-6) {
                        1.0f
                    } else if (abs(t) == 0.5f / alpha) {
                        (PI / 4) * sinc(0.5f / alpha).toFloat()
                    } else {
                        sinc(t) * cos(PI * alpha * t) / (1 - 4 * alpha * alpha * t * t)
                    }
                    sum += samples[sampleIndex] * h
                }
            }
            
            filteredSamples[i] = sum * 0.8f // Scale to prevent clipping
        }
        
        return filteredSamples
    }
    
    private fun sinc(x: Float): Float {
        return if (abs(x) < 1e-6) 1.0f else (sin(PI * x) / (PI * x)).toFloat()
    }
    
    // Reset modulator state
    fun reset() {
        phase = 0.0
        symbolBuffer.clear()
    }
}