import wave
import threading
import queue
import time
import os
from datetime import datetime
from PyQt5.QtCore import QObject, pyqtSignal

# Try to import PyAudio, but handle if it's not available
try:
    import pyaudio
    PYAUDIO_AVAILABLE = True
except ImportError:
    PYAUDIO_AVAILABLE = False
    print("Warning: PyAudio not installed. Audio playback will be disabled.")
    print("To enable playback, install with: sudo dnf install python3-devel portaudio-devel && pip install pyaudio")

class AudioPlayer(QObject):
    playback_started = pyqtSignal(int)  # client_id
    playback_stopped = pyqtSignal(int)  # client_id
    recording_saved = pyqtSignal(int, str)  # client_id, filepath
    
    def __init__(self):
        super().__init__()
        self.audio = None
        self.streams = {}  # client_id -> stream
        self.buffers = {}  # client_id -> queue
        self.recording_buffers = {}  # client_id -> list of audio data
        self.recording_enabled = {}  # client_id -> bool
        self.playback_enabled = {}  # client_id -> bool
        self.sample_rate = 8000
        self.channels = 1
        self.chunk_size = 320  # 40ms at 8kHz
        self.debug_callback = None
        self.actual_sample_rate = 8000  # Will be updated if needed
        
        if PYAUDIO_AVAILABLE:
            try:
                self.audio = pyaudio.PyAudio()
            except Exception as e:
                self.debug(f"Failed to initialize PyAudio: {e}")
                self.audio = None
        else:
            self.audio = None
            self.debug("PyAudio not available - playback disabled, recording still works")
    
    def debug(self, message):
        if self.debug_callback:
            self.debug_callback(f"[AudioPlayer] {message}")
        else:
            print(f"[AudioPlayer] {message}")
    
    def set_debug_callback(self, callback):
        self.debug_callback = callback
    
    def start_playback(self, client_id):
        """Start audio playback for a client"""
        if not self.audio:
            self.debug("Audio playback not available - PyAudio not installed")
            self.debug("To enable: sudo dnf install python3-devel portaudio-devel && pip install pyaudio")
            return False
            
        if client_id in self.streams:
            self.debug(f"Playback already active for client {client_id}")
            return False
        
        try:
            # Create buffer for this client
            self.buffers[client_id] = queue.Queue(maxsize=100)  # Limit queue size
            self.playback_enabled[client_id] = True
            
            # Try different sample rates if 8000 Hz fails
            sample_rates = [8000, 16000, 44100, 48000]
            stream = None
            
            for rate in sample_rates:
                try:
                    # Adjust buffer size based on sample rate
                    buffer_frames = int(640 * rate / 8000)  # Scale buffer size
                    
                    # Create audio stream with callback for continuous playback
                    def audio_callback(in_data, frame_count, time_info, status):
                        if status:
                            self.debug(f"Playback status for client {client_id}: {status}")
                        
                        # Get audio data from buffer
                        audio_data = b''
                        bytes_needed = frame_count * 2  # 16-bit samples
                        
                        # Try to get enough data for the requested frame count
                        while len(audio_data) < bytes_needed:
                            try:
                                chunk = self.buffers[client_id].get_nowait()
                                
                                # Resample if needed
                                if self.actual_sample_rate != self.sample_rate:
                                    chunk = self._resample_audio(chunk, self.sample_rate, self.actual_sample_rate)
                                
                                audio_data += chunk
                            except queue.Empty:
                                # No more data available, pad with silence
                                if len(audio_data) < bytes_needed:
                                    silence = b'\x00' * (bytes_needed - len(audio_data))
                                    audio_data += silence
                                break
                        
                        # Trim to exact size if we got too much
                        if len(audio_data) > bytes_needed:
                            # Put extra back in queue
                            extra = audio_data[bytes_needed:]
                            try:
                                self.buffers[client_id].put_nowait(extra)
                            except queue.Full:
                                pass
                            audio_data = audio_data[:bytes_needed]
                        
                        return (audio_data, pyaudio.paContinue)
                    
                    # Try to create stream with current sample rate
                    stream = self.audio.open(
                        format=pyaudio.paInt16,
                        channels=self.channels,
                        rate=rate,
                        output=True,
                        frames_per_buffer=buffer_frames,
                        stream_callback=audio_callback
                    )
                    
                    self.actual_sample_rate = rate
                    if rate != self.sample_rate:
                        self.debug(f"Using sample rate {rate} Hz (resampling from {self.sample_rate} Hz)")
                    
                    break  # Success!
                    
                except Exception as e:
                    if rate == sample_rates[-1]:  # Last attempt
                        raise e
                    else:
                        self.debug(f"Sample rate {rate} Hz failed, trying next...")
                        continue
            
            if not stream:
                raise Exception("Could not create audio stream with any sample rate")
            
            self.streams[client_id] = stream
            stream.start_stream()
            
            self.debug(f"Started callback-based playback for client {client_id} at {self.actual_sample_rate} Hz")
            self.playback_started.emit(client_id)
            return True
            
        except Exception as e:
            self.debug(f"Failed to start playback for client {client_id}: {e}")
            self.playback_enabled[client_id] = False
            if client_id in self.buffers:
                del self.buffers[client_id]
            return False
    
    def stop_playback(self, client_id):
        """Stop audio playback for a client"""
        if client_id not in self.streams:
            return
        
        self.playback_enabled[client_id] = False
        
        # Stop and close stream
        if client_id in self.streams:
            try:
                self.streams[client_id].stop_stream()
                self.streams[client_id].close()
            except:
                pass
            del self.streams[client_id]
        
        # Clear buffer
        if client_id in self.buffers:
            # Clear any remaining data
            while not self.buffers[client_id].empty():
                try:
                    self.buffers[client_id].get_nowait()
                except:
                    break
            del self.buffers[client_id]
        
        self.debug(f"Stopped playback for client {client_id}")
        self.playback_stopped.emit(client_id)
    
    def add_audio_data(self, client_id, pcm_data):
        """Add audio data to playback buffer"""
        # Initialize frame counter for debug logging
        if not hasattr(self, '_frame_count'):
            self._frame_count = {}
        if client_id not in self._frame_count:
            self._frame_count[client_id] = 0
        self._frame_count[client_id] += 1
        
        # Only log occasionally to avoid spam
        if self._frame_count[client_id] == 1 or self._frame_count[client_id] % 25 == 0:
            self.debug(f"Client {client_id} audio frame #{self._frame_count[client_id]}: {len(pcm_data)} bytes")
        
        if client_id in self.buffers:
            try:
                # Use put_nowait to avoid blocking
                self.buffers[client_id].put_nowait(pcm_data)
                if self._frame_count[client_id] == 1:
                    self.debug(f"Client {client_id} buffer started, queue size: {self.buffers[client_id].qsize()}")
            except queue.Full:
                # Buffer is full, drop oldest data to make room
                try:
                    self.buffers[client_id].get_nowait()  # Remove oldest
                    self.buffers[client_id].put_nowait(pcm_data)  # Add newest
                    if self._frame_count[client_id] % 50 == 0:  # Log occasionally
                        self.debug(f"Client {client_id} buffer overflow, dropping old data")
                except:
                    pass
        else:
            if self._frame_count[client_id] == 1:
                self.debug(f"Client {client_id} has no buffer (playback not started?)")
        
        # Add to recording buffer if recording
        if self.recording_enabled.get(client_id, False):
            if client_id not in self.recording_buffers:
                self.recording_buffers[client_id] = []
            self.recording_buffers[client_id].append(pcm_data)
    
    
    def start_recording(self, client_id):
        """Start recording received audio"""
        self.recording_enabled[client_id] = True
        self.recording_buffers[client_id] = []
        self.debug(f"Started recording for client {client_id}")
    
    def stop_recording(self, client_id, save_path=None):
        """Stop recording and optionally save to file"""
        if not self.recording_enabled.get(client_id, False):
            return None
        
        self.recording_enabled[client_id] = False
        
        if client_id not in self.recording_buffers:
            return None
        
        audio_data = self.recording_buffers[client_id]
        
        if not audio_data:
            self.debug(f"No audio data recorded for client {client_id}")
            return None
        
        # Generate filename if not provided
        if not save_path:
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            save_path = f"wav/received_client{client_id}_{timestamp}.wav"
        
        # Ensure directory exists
        save_dir = os.path.dirname(save_path)
        if save_dir:
            os.makedirs(save_dir, exist_ok=True)
        
        try:
            # Combine all audio chunks
            combined_audio = b''.join(audio_data)
            
            # Save as WAV file
            with wave.open(save_path, 'wb') as wav_file:
                wav_file.setnchannels(self.channels)
                wav_file.setsampwidth(2)  # 16-bit
                wav_file.setframerate(self.sample_rate)  # Always save at original 8kHz
                wav_file.writeframes(combined_audio)
            
            self.debug(f"Saved recording for client {client_id} to {save_path}")
            self.recording_saved.emit(client_id, save_path)
            
            # Clear recording buffer
            del self.recording_buffers[client_id]
            
            return save_path
            
        except Exception as e:
            self.debug(f"Failed to save recording for client {client_id}: {e}")
            return None
    
    def _resample_audio(self, audio_data, from_rate, to_rate):
        """Simple linear resampling of audio data"""
        if from_rate == to_rate:
            return audio_data
        
        import struct
        
        # Convert bytes to samples
        samples = struct.unpack(f'{len(audio_data)//2}h', audio_data)
        
        # Calculate resampling ratio
        ratio = to_rate / from_rate
        new_length = int(len(samples) * ratio)
        
        # Simple linear interpolation
        resampled = []
        for i in range(new_length):
            # Find position in original samples
            pos = i / ratio
            idx = int(pos)
            frac = pos - idx
            
            if idx < len(samples) - 1:
                # Linear interpolation between samples
                sample = int(samples[idx] * (1 - frac) + samples[idx + 1] * frac)
            else:
                # Use last sample
                sample = samples[-1] if samples else 0
            
            resampled.append(sample)
        
        # Convert back to bytes
        return struct.pack(f'{len(resampled)}h', *resampled)
    
    def cleanup(self):
        """Clean up audio resources"""
        # Stop all playback
        for client_id in list(self.streams.keys()):
            self.stop_playback(client_id)
        
        # Terminate PyAudio
        if self.audio:
            self.audio.terminate()
            self.audio = None