from PyQt5.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLabel, QFrame,
    QGridLayout, QProgressBar
)
from PyQt5.QtCore import Qt, pyqtSignal, QTimer
from PyQt5.QtGui import QFont, QPalette, QColor, QPainter, QBrush, QLinearGradient
import math

class SignalStrengthWidget(QWidget):
    """Custom widget to display signal strength bars"""
    def __init__(self, parent=None):
        super().__init__(parent)
        self.signal_strength = -70  # dBm
        self.setMinimumSize(60, 40)
        self.setMaximumSize(80, 50)
        
    def set_signal_strength(self, dbm):
        self.signal_strength = dbm
        self.update()
        
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        # Calculate number of bars based on signal strength
        # -50 dBm or better = 5 bars
        # -60 dBm = 4 bars
        # -70 dBm = 3 bars
        # -80 dBm = 2 bars
        # -90 dBm = 1 bar
        # < -90 dBm = 0 bars
        
        if self.signal_strength >= -50:
            active_bars = 5
        elif self.signal_strength >= -60:
            active_bars = 4
        elif self.signal_strength >= -70:
            active_bars = 3
        elif self.signal_strength >= -80:
            active_bars = 2
        elif self.signal_strength >= -90:
            active_bars = 1
        else:
            active_bars = 0
            
        bar_width = 10
        bar_spacing = 3
        max_height = 35
        
        for i in range(5):
            x = i * (bar_width + bar_spacing) + 5
            bar_height = (i + 1) * 7
            y = max_height - bar_height + 10
            
            if i < active_bars:
                # Active bar - gradient from green to yellow to red based on strength
                if self.signal_strength >= -60:
                    color = QColor(76, 175, 80)  # Green
                elif self.signal_strength >= -75:
                    color = QColor(255, 193, 7)  # Amber
                else:
                    color = QColor(244, 67, 54)  # Red
            else:
                # Inactive bar
                color = QColor(60, 60, 60)
                
            painter.fillRect(x, y, bar_width, bar_height, color)

class GSMStatusWidget(QFrame):
    """Widget to display GSM network status and parameters"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setObjectName("gsmStatusWidget")
        self.setFrameStyle(QFrame.StyledPanel)
        
        # Default settings
        self.settings = {
            'signal_strength': -70,
            'signal_quality': 75,
            'noise_level': 10,
            'codec_mode': 'AMR-NB',
            'bitrate': 12.2,
            'packet_loss': 0,
            'jitter': 20,
            'latency': 100,
            'fading_enabled': False,
            'fading_speed': 'slow',
            'interference_enabled': False,
            'handover_enabled': False
        }
        
        self.init_ui()
        self.update_display()
        
    def init_ui(self):
        """Initialize the UI components"""
        main_layout = QVBoxLayout()
        main_layout.setSpacing(10)
        main_layout.setContentsMargins(15, 15, 15, 15)
        
        # Title
        title_label = QLabel("📡 GSM Network Status")
        title_label.setObjectName("gsmStatusTitle")
        title_label.setAlignment(Qt.AlignCenter)
        main_layout.addWidget(title_label)
        
        # Main status grid
        status_grid = QGridLayout()
        status_grid.setSpacing(12)
        
        # Row 0: Signal strength visualization
        status_grid.addWidget(QLabel("Signal:"), 0, 0)
        self.signal_widget = SignalStrengthWidget()
        status_grid.addWidget(self.signal_widget, 0, 1)
        self.signal_dbm_label = QLabel("-70 dBm")
        self.signal_dbm_label.setObjectName("signalDbmLabel")
        status_grid.addWidget(self.signal_dbm_label, 0, 2)
        
        # Row 1: Signal quality bar
        status_grid.addWidget(QLabel("Quality:"), 1, 0)
        self.quality_bar = QProgressBar()
        self.quality_bar.setTextVisible(True)
        self.quality_bar.setRange(0, 100)
        self.quality_bar.setObjectName("qualityBar")
        status_grid.addWidget(self.quality_bar, 1, 1, 1, 2)
        
        # Row 2: Noise level
        status_grid.addWidget(QLabel("Noise:"), 2, 0)
        self.noise_bar = QProgressBar()
        self.noise_bar.setTextVisible(True)
        self.noise_bar.setRange(0, 50)
        self.noise_bar.setObjectName("noiseBar")
        status_grid.addWidget(self.noise_bar, 2, 1, 1, 2)
        
        main_layout.addLayout(status_grid)
        
        # Separator
        separator1 = QFrame()
        separator1.setFrameShape(QFrame.HLine)
        separator1.setObjectName("gsmSeparator")
        main_layout.addWidget(separator1)
        
        # Codec info section
        codec_layout = QHBoxLayout()
        codec_layout.setSpacing(15)
        
        codec_icon = QLabel("🎤")
        codec_layout.addWidget(codec_icon)
        
        self.codec_label = QLabel("AMR-NB @ 12.2 kbps")
        self.codec_label.setObjectName("codecLabel")
        codec_layout.addWidget(self.codec_label)
        codec_layout.addStretch()
        
        main_layout.addLayout(codec_layout)
        
        # Network conditions section
        network_grid = QGridLayout()
        network_grid.setSpacing(8)
        
        # Packet loss indicator
        network_grid.addWidget(QLabel("📉"), 0, 0)
        self.packet_loss_label = QLabel("Loss: 0%")
        self.packet_loss_label.setObjectName("networkParam")
        network_grid.addWidget(self.packet_loss_label, 0, 1)
        
        # Jitter indicator
        network_grid.addWidget(QLabel("📊"), 0, 2)
        self.jitter_label = QLabel("Jitter: 20ms")
        self.jitter_label.setObjectName("networkParam")
        network_grid.addWidget(self.jitter_label, 0, 3)
        
        # Latency indicator
        network_grid.addWidget(QLabel("⏱"), 1, 0)
        self.latency_label = QLabel("Latency: 100ms")
        self.latency_label.setObjectName("networkParam")
        network_grid.addWidget(self.latency_label, 1, 1)
        
        # Features status
        network_grid.addWidget(QLabel("🌊"), 1, 2)
        self.features_label = QLabel("Standard")
        self.features_label.setObjectName("networkParam")
        network_grid.addWidget(self.features_label, 1, 3)
        
        main_layout.addLayout(network_grid)
        
        # Connection status
        separator2 = QFrame()
        separator2.setFrameShape(QFrame.HLine)
        separator2.setObjectName("gsmSeparator")
        main_layout.addWidget(separator2)
        
        self.connection_status = QLabel("🟢 Connected to GSM Network")
        self.connection_status.setObjectName("connectionStatus")
        self.connection_status.setAlignment(Qt.AlignCenter)
        main_layout.addWidget(self.connection_status)
        
        main_layout.addStretch()
        self.setLayout(main_layout)
        
        # Apply styling
        self.setStyleSheet("""
            #gsmStatusWidget {
                background-color: #2A2A2A;
                border: 2px solid #0078D4;
                border-radius: 10px;
            }
            #gsmStatusTitle {
                font-size: 16px;
                font-weight: bold;
                color: #00A2E8;
                padding: 5px;
            }
            #signalDbmLabel {
                font-size: 14px;
                font-weight: bold;
                color: #4CAF50;
            }
            #qualityBar {
                background-color: #1E1E1E;
                border: 1px solid #444;
                border-radius: 3px;
                text-align: center;
                color: white;
            }
            #qualityBar::chunk {
                background-color: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 #4CAF50, stop:0.5 #8BC34A, stop:1 #CDDC39);
                border-radius: 3px;
            }
            #noiseBar {
                background-color: #1E1E1E;
                border: 1px solid #444;
                border-radius: 3px;
                text-align: center;
                color: white;
            }
            #noiseBar::chunk {
                background-color: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 #FFC107, stop:0.5 #FF9800, stop:1 #FF5722);
                border-radius: 3px;
            }
            #codecLabel {
                font-size: 13px;
                color: #E0E0E0;
                font-weight: bold;
            }
            #networkParam {
                font-size: 12px;
                color: #B0B0B0;
            }
            #gsmSeparator {
                background-color: #444;
                max-height: 1px;
                margin: 5px 0;
            }
            #connectionStatus {
                font-size: 13px;
                color: #4CAF50;
                font-weight: bold;
                padding: 5px;
            }
            QLabel {
                color: #E0E0E0;
            }
        """)
        
    def update_settings(self, settings):
        """Update the displayed settings"""
        self.settings = settings
        self.update_display()
        
    def update_display(self):
        """Update all display elements based on current settings"""
        # Update signal strength
        self.signal_widget.set_signal_strength(self.settings['signal_strength'])
        self.signal_dbm_label.setText(f"{self.settings['signal_strength']} dBm")
        
        # Color code the dBm label
        if self.settings['signal_strength'] >= -60:
            self.signal_dbm_label.setStyleSheet("color: #4CAF50;")  # Green
        elif self.settings['signal_strength'] >= -75:
            self.signal_dbm_label.setStyleSheet("color: #FFC107;")  # Amber
        else:
            self.signal_dbm_label.setStyleSheet("color: #FF5722;")  # Red
            
        # Update quality bar
        self.quality_bar.setValue(self.settings['signal_quality'])
        self.quality_bar.setFormat(f"{self.settings['signal_quality']}%")
        
        # Update noise bar
        self.noise_bar.setValue(self.settings['noise_level'])
        self.noise_bar.setFormat(f"{self.settings['noise_level']}%")
        
        # Update codec info
        self.codec_label.setText(f"{self.settings['codec_mode']} @ {self.settings['bitrate']} kbps")
        
        # Update network parameters
        self.packet_loss_label.setText(f"Loss: {self.settings['packet_loss']}%")
        self.jitter_label.setText(f"Jitter: {self.settings['jitter']}ms")
        self.latency_label.setText(f"Latency: {self.settings['latency']}ms")
        
        # Update features
        features = []
        if self.settings['fading_enabled']:
            features.append(f"Fading({self.settings['fading_speed']})")
        if self.settings['interference_enabled']:
            features.append("Interference")
        if self.settings['handover_enabled']:
            features.append("Handover")
            
        if features:
            self.features_label.setText(", ".join(features))
        else:
            self.features_label.setText("Standard")
            
        # Update connection status based on signal quality
        if self.settings['signal_quality'] >= 80:
            self.connection_status.setText("🟢 Excellent Connection")
            self.connection_status.setStyleSheet("color: #4CAF50;")
        elif self.settings['signal_quality'] >= 60:
            self.connection_status.setText("🟡 Good Connection")
            self.connection_status.setStyleSheet("color: #FFC107;")
        elif self.settings['signal_quality'] >= 40:
            self.connection_status.setText("🟠 Fair Connection")
            self.connection_status.setStyleSheet("color: #FF9800;")
        else:
            self.connection_status.setText("🔴 Poor Connection")
            self.connection_status.setStyleSheet("color: #FF5722;")